------------------------------------------
-- Setting up libraries
------------------------------------------

local AddonName = "iMoney"
iMoney = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0");

local QTip = LibStub("LibQTip-1.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

local Tooltip;

iMoney.Broker = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",

	OnEnter = function(anchor)
		Tooltip = QTip:Acquire(iMoney);
		Tooltip:SetColumnLayout(2, "LEFT", "RIGHT");
		Tooltip:SetAutoHideDelay(0.01, anchor);
		Tooltip:SmartAnchorTo(anchor);
		iMoney:UpdateTooltip();
		Tooltip:Show();
	end,
});

------------------------------------------
-- Variables, functions and colors
------------------------------------------

local COLOR_GOLD = "|cfffed100%s|r";
local COLOR_RED  = "|cffff0000%s|r";
local COLOR_GREEN= "|cff00ff00%s|r";

iMoney.Gold = 0;

local CharName = GetUnitName("player", false);

local ClassColors = {};
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end

------------------------------------------
-- OnInitialize and OnEnable
------------------------------------------

function iMoney:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("iMoneyDB", {realm={today = "",chars={}}}, true).realm;
	
	if( not self.db.chars[CharName] ) then
		self.db.chars[CharName] = {
			gold = 0,
			class = UnitClass("player"),
			today = 0,
		};
	end
	
	self:RegisterEvent("PLAYER_MONEY", "UpdateMoney");
end

function iMoney:OnEnable()
	self:UpdateMoney();
	
	if( self.db.chars[CharName].today == 0 and self.db.chars[CharName].today ~= self.db.chars[CharName].gold ) then
		self.db.chars[CharName].today = self.db.chars[CharName].gold;
	end
	
	local today = date("%y-%m-%d");
	if( today ~= self.db.today ) then
		self.db.today = today;
		
		for k, v in pairs(self.db.chars) do
			self.db.chars[k].today = self.db.chars[k].gold;
		end
	end
end

------------------------------------------
-- MoneyString and UpdateMoney
------------------------------------------

function iMoney:MoneyString(gold, encolor)
	local str;

	if( encolor ) then
		if( gold < 0 ) then
			str = (COLOR_RED):format("-"..GetCoinTextureString(-gold, 0));
		elseif( gold > 0 ) then
			str = (COLOR_GREEN):format(GetCoinTextureString(gold, 0));
		else
			str = GetCoinTextureString(gold, 0);
		end
	else
		if( gold < 0 ) then
			str = "-"..GetCoinTextureString(-gold, 0);
		else
			str = GetCoinTextureString(gold, 0);
		end
	end
	
	return str;
end

function iMoney:UpdateMoney()
	local gold = GetMoney();
	
	self.db.chars[CharName].gold = gold;
	self.Broker.text = self:MoneyString(gold);
	
	iMoney.Gold = gold;
end

------------------------------------------
-- UpdateTooltip
------------------------------------------

local function iMoneyClickLine(_, name, button)
	if( button == "RightButton" and name ~= CharName ) then
		iMoney.db.chars[name] = nil;
		iMoney:UpdateTooltip();
	end
end

function iMoney:UpdateTooltip()
	Tooltip:Clear();
	
	Tooltip:AddLine(
		(COLOR_GOLD):format(L["Today Session"]),
		self:MoneyString(self.Gold - self.db.chars[CharName].today, true)
	);
	Tooltip:AddLine(" ");
	
	local total = 0;
	
	for k, v in pairs(self.db.chars) do
		local line = Tooltip:AddLine(
			("%s%s|r"):format(ClassColors[v.class], k),
			self:MoneyString(v.gold)
		);
		total = total + v.gold;
	
		Tooltip:SetLineScript(line, "OnMouseDown", iMoneyClickLine, k);
	end
	
	Tooltip:AddLine(" ");
	Tooltip:AddLine(
		(COLOR_GOLD):format(L["Total Gold"]),
		self:MoneyString(total)
	);
end